# zib -- zzznowman's IRC bot
# Copyright (C) 2006  Fabian Pietsch <fabian@zzznowman.dyndns.org>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Helper functions

package ZIB;
use strict;
use warnings;
use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS %loglevels $loglevel);
use POSIX qw(strftime);
require Exporter;

$VERSION = 20060604.1;
@ISA = qw(Exporter);

@EXPORT_OK = qw(%loglevels $loglevel logsym2level setloglevel
                log_checkstate log_msg error warning notice info debug
                split_line raw2nuh raw2nick raw2user raw2host);

%EXPORT_TAGS = (
  all => \@EXPORT_OK,

  logging      => [qw(error warning notice info debug)],
  all_logging  => [qw(%loglevels $loglevel logsym2level setloglevel
                      log_checkstate log_msg error warning notice info debug)],

  conversions     => [qw(split_line raw2nick raw2user raw2host)],
  all_conversions => [qw(split_line raw2nuh raw2nick raw2user raw2host)]
);


%loglevels = (ERROR => 1, WARNING => 2, NOTICE => 3, INFO => 4, DEBUG => 5);
$loglevel = $loglevels{INFO};

my $log_prevmday;


# Logging

sub logsym2level($)
{
	my ($sym) = @_;
	return ($sym =~ /^\d+$/) ? $sym : ($loglevels{$sym} || 0);
}

sub setloglevel($)
{
	my ($newsym) = @_;
	my $oldlevel = $loglevel;

	if (defined($newsym))
	{
		$loglevel = logsym2level($newsym);
	}

	return $oldlevel;
}

# This might later be used to open/close log files etc.
# Should be called from log_msg() before logging,
# and regularly from somewhere else. (main loop?)
sub log_checkstate()
{
	my @ltime = localtime();
	my $mday = $ltime[3];

	unless (defined($log_prevmday))
	{
		# Log this before logging anything else.
		printf("<0> Logging started %s\n",
		       strftime("%Y-%m-%d %H:%M:%S", @ltime));
	}
	elsif ($log_prevmday != $mday)
	{
		# Log this on day change.
		printf("<0> Day changed to %s\n",
		       strftime("%Y-%m-%d", @ltime));
	}
	else
	{
		return;
	}

	$log_prevmday = $mday;
}

sub log_msg($$@)
{
	my ($logsym, $format, @args) = @_;
	my $msglevel = logsym2level($logsym);
	return if ($msglevel > $loglevel);

	log_checkstate();

	printf("<%d> %s" . $format . "\n", $msglevel,
	       strftime("[%H:%M:%S] ", localtime()), @args);
}

sub error($@)   { unshift(@_, 'ERROR');   &log_msg; }
sub warning($@) { unshift(@_, 'WARNING'); &log_msg; }
sub notice($@)  { unshift(@_, 'NOTICE');  &log_msg; }
sub info($@)    { unshift(@_, 'INFO');    &log_msg; }
sub debug($@)   { unshift(@_, 'DEBUG');   &log_msg; }


# Input conversion

sub split_line($)
{
	my ($line) = @_;

	# Remove whitespace.
	chomp($line);
	$line =~ s/^\s+|\s+$//g;
	$line =~ s/\s+/ /g;

	# Split into words.
	return split(/ /, $line);
}

sub raw2nuh($)
{
	my ($raw) = @_;
	my ($nick, $user, $host) = $raw;

	if ($raw =~ /^([^!@]*)(?:!([^@]*))?(?:@(.*))?$/)
	{
		$nick = $1;
		$user = $2;
		$host = $3;
	}

	return ($nick, $user, $host);
}

sub raw2nick($)
{
	return (&raw2nuh)[0];
}

sub raw2user($)
{
	return (&raw2nuh)[1];
}

sub raw2host($)
{
	return (&raw2nuh)[2];
}

1;
